//=============================================================================
//    
//=============================================================================

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

#include "xml_lib.h"
#include "gen_ppp.h"


//-----------------------------------------------------------------------------
//     
FILE* output = stdout;
//,            
bool sw_defh_ignore = false;


//-----------------------------------------------------------------------------
//     ( /)
class CResourceManager {
	//       
	// output_file != NULL,      output
	FILE* output_file;
	//    ,   O2M makefile
	char* path2mk;
public:
	//      .2ml
	const char* path2ml;
	//     ( )
	const char* pathPPP;
	//      stdout
	bool SetOutput(const char* fname);
	//   .2ml      
	void SetPath2mlByFName(const char* fname);
	//   .2ml    
	void SetPath2mlByPath(const char* path);
	// ( )
	CResourceManager() : output_file(NULL),
		path2mk(NULL),
		path2ml(NULL),
		pathPPP(NULL)
	{};
	// ( )
	~CResourceManager()
	{
		// ,   O2M makefile
		delete[] path2mk;
		//      (.  '-f')
		if (output_file) {
			output = stdout;
			if (fclose(output_file)) fprintf(output, "FILE I/O: Cannot close file, specified by switch -f\n");
		}
	};
} ResourceManager;


//-----------------------------------------------------------------------------
//      stdout
bool CResourceManager::SetOutput(const char *fname)
{
	//     
	output_file = fopen(fname, "w");
	if (!output_file) {
		fprintf(output, "FILE I/O: Cannot open file %s for writing\n", fname);
		return false;
	}
	//     
	output = output_file;
	return true;
}


//-----------------------------------------------------------------------------
//   .2ml    
void CResourceManager::SetPath2mlByFName(const char* fname)
{
	int i = strlen(fname) - 1;
	while (i >= 0 && '/' != fname[i] && '\\' != fname[i]) --i;
	if (i < 0) {
		path2mk = new char [1];
		path2mk[0] = '\0';
	} else {
		++i;
		path2mk = new char[i + 1];
		strncpy(path2mk, fname, i);
		path2mk[i] = '\0';
	}
	path2ml = path2mk;
}


//-----------------------------------------------------------------------------
//   .2ml    
void CResourceManager::SetPath2mlByPath(const char* path)
{
	int len = strlen(path);
	if (len > 0 && '/' != path[len - 1] && '\\' != path[len - 1]) {
		path2mk = new char[len + 2];
		strcpy(path2mk, path);
		path2mk[len] = '/';
		path2mk[len + 1] = '\0';
		path2ml = path2mk;
	} else
		path2ml = path;
}


//-----------------------------------------------------------------------------
//   
int main(int argc, char* argv[])
{
	// Link2M
	const char* const Version = "1.8.12";

	// ,    
	const int ErrExitCode = 1;

	//   ,    
	bool sw_about = true;
	bool sw_version = false;
	bool sw_help = false;
	bool sw_arg_err = false;

	//   
	if (argc > 1) {
		for (int i = 1; i < argc; i++)
			if ('-' == argv[i][0] || '/' == argv[i][0]) {
				//   
				char sw = toupper(argv[i][1]);

				//  'a'
				if ('A' == sw && '\0' == argv[i][2]) sw_about = false;
				else

				//  'd'
				if ('D' == sw && '\0' == argv[i][2]) sw_defh_ignore = true;
				else

				//  'i'
				if ('I' == sw && '\0' == argv[i][2]) {
					//     
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found path after -i in command line\n");
						sw_arg_err = true;
						break;
					}
					//     .2ml 
					ResourceManager.SetPath2mlByPath(argv[i]);
				} else

				//  'o'
				if ('O' == sw && '\0' == argv[i][2]) {
					//     
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found path after -o in command line\n");
						sw_arg_err = true;
						break;
					}
					//     .2ml 
					ResourceManager.pathPPP = argv[i];
				} else

				//  'f'
				if ('F' == sw && '\0' == argv[i][2]) {
					//      
					if (++i >= argc) {
						fprintf(output, "ERROR: Not found file name after -f in command line\n");
						sw_arg_err = true;
						break;
					}
					//        
					if (!ResourceManager.SetOutput(argv[i])) return ErrExitCode;
				} else

				//  'v'
				if ('V' == sw && '\0' == argv[i][2]) sw_version = true;
				else

				//  'h'
				if (('H' == sw || '?' == sw) && '\0' == argv[i][2])
					sw_help = true;
				else {
					fprintf(output, "ERROR: Unknown command line option %s\n", argv[i]);
					sw_arg_err = true;
					break;
				}
			}
	} else sw_help = true;

	//   ( )
	if (sw_version) {
		fprintf(output, "Link2M version %s\n", Version);
		return 0;
	}
	//    -  
	if (!sw_help && !sw_arg_err && (argc < 2 || '-' == argv[1][0] || '/' == argv[1][0])) {
		fprintf(output, "ERROR: Not specified O2M makefile\n");
		sw_arg_err = true;
	}
	//   ( )
	if (sw_about) fprintf(output, "\nLink2M (O2M linker) v%s\nCopyright(c) Schwetz Dmitry, 2004\n\n", Version);
	if (sw_help || sw_arg_err) {
		fprintf(output, "Usage:      Link2M <O2M_makefile> [-<switch>...]\n\n<Examples>\n  Link2M _O2M_make.2mk\n  Link2M \"_O2M_make.2mk\" -f \"log.txt\" -a\n\n<Switches>\n");
		fprintf(output, "  i <path>  Set path for input files (equal to O2M makefile path by default)\n");
		fprintf(output, "  o <path>  Set path for output files (equal to O2M makefile path by default)\n");
		fprintf(output, "  d         Do not treat as error insufficiency of handlers for common\n             procedure without default handler\n");
		fprintf(output, "  a         Disable about info\n");
		fprintf(output, "  f <file>  Specify file for output messages instead of stdout\n");
		fprintf(output, "  v         Version info\n");
		fprintf(output, "  h,?       This help\n");
		return sw_arg_err ? ErrExitCode : 0;
	}

	// xml      (_O2M_make.2mk)
	CXMLFile xmlFile;
	switch (xmlFile.Load(argv[1])) {
	case EXMLNoError:
		//    ( - Project)
		if (strcmp("Project", xmlFile.XMLEntry->TagName)) {
			fprintf(output, "ERROR: Incorrect O2M makefile (\"%s\") content\n", argv[1]);
			goto fault_exit;
		}
		break;
	case EXMLFormat:
		fprintf(output, "ERROR: Incorrect O2M makefile (\"%s\") format\n", argv[1]);
		goto fault_exit;
	case EXMLOpen:
		fprintf(output, "ERROR: Cannot open file \"%s\" for reading\n", argv[1]);
		goto fault_exit;
	case EXMLClose:
		fprintf(output, "ERROR: Cannot close file \"%s\"\n", argv[1]);
		goto fault_exit;
	default:
		fprintf(output, "ERROR: UNKNOWN INTERNAL ERROR\n");
		goto fault_exit;
	}

	//   .2ml ,        
	if (!ResourceManager.path2ml) ResourceManager.SetPath2mlByFName(argv[1]);

	//   _O2M_ppp.* 
	if (gen_ppp_files(xmlFile.XMLEntry, ResourceManager.path2ml, ResourceManager.pathPPP)) {
		fprintf(output, "Link2M - OK\n");
		return 0;
	}

fault_exit:
	fprintf(output, "Link2M - FAILED\n");
	return ErrExitCode;
}
